import bpy

bl_info = {
    "name": "Scene Node Groups",
    "author": "DBG",
    "version": (1, 0),
    "blender": (3, 40, 0),
    "location": "View3D > UI > Scene",
    "description": "List All Node Groups in the Scene",
    "warning": "",
    "doc_url": "",
    "category": "Node Groups",
}

# Node Groups 
_scene_node_groups = []

## ENUM PROPERTY FUNCTIONS

def get_all_node_groups(self, context):
    items=('GRP', 'None', 'None Node Groups in the Scene')
    # Dr. Sybren Explains :https://devtalk.blender.org/t/can-bpy-props-be-used-for-dynamic-lists/10130/10
    global _scene_node_groups  
    _scene_node_groups.clear()
    groups = list(bpy.data.node_groups)
    if groups:
        _scene_node_groups = [('GRP_'+ str(index), str(node_group.name), "") for index, node_group in enumerate(groups)]
        return _scene_node_groups
    else:
        _scene_node_groups = [(items)]

    return _scene_node_groups


def get_all_node_groups_(self, context):
    items=('GRP', 'None', 'None Node Groups in the Scene')
    _scene_node_groups.clear()

    groups = list(bpy.data.node_groups)
    if groups:
        for index, node_group in enumerate(groups):
            unique_ident = 'GRP_'+ str(index)
            _scene_node_groups.append((unique_ident, node_group.name, node_group.bl_idname))
    else:
        _scene_node_groups.append(items)

    return _scene_node_groups


def get_node_group_enum(self):
    return self

def set_node_group_enum(self, value):
    return None
    
def update_node_group_enum(self, context):
    return None

## END ENUM PROPERTY FUNCTIONS

## UI LIST FUNCTIONS

def get_all_data_node_groups(self):
    '''Get callback to create the list of all groups in the scene.
       Used by the Boolean Property
        :arg self: Scene
        :type self: Scene
    '''
    update = set(node_group for node_group in bpy.data.node_groups)  #.symmetric_difference(self.node_groups)
    update = [node_group for node_group in bpy.data.node_groups]  #.symmetric_difference(self.node_groups)
    if update:
        set_all_data_node_groups(self, True)
    return False

def set_all_data_node_groups(self, value):
    '''Set callback to create the list of all groups in the scene.
       Used by the Boolean Property
        :arg self: Scene
        :type self: Scene
    '''
    if value:
        self.node_groups_list.clear() # update (or get) instead?
        for node_group in self.node_groups:
            n_g = self.node_groups_list.add()
            n_g.name = node_group.name
            n_g.node_group = node_group
            
def get_scene_node_groups(self):
    '''Items calback for the Scene node_group property
         :arg self: Scene
        :type self: Scene
    '''
    return set(node_group for node_group in bpy.data.node_groups)

## END UI LIST FUNCTIONS

## PROPERTY GROUP

class SceneNodeGroupsPropGroup(bpy.types.PropertyGroup):
    #def get_name(self):
    #    return getattr(self.node_groups, "name", "")
    #name: bpy.props.StringProperty()
    node_group : bpy.props.PointerProperty(type=bpy.types.NodeTree)

## END ROPERTY GROUP
    
## UI LIST
    
class CUSTOM_UL_scene_node_groups(bpy.types.UIList):
    
    def draw_item(self, _context, layout, _data, item, icon, active_data, active_propname, index):
        node_group_slot = item
        node_group = node_group_slot.node_group

        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            if node_group:
                layout.prop(node_group, "name", text="", emboss=False, icon_value=layout.icon(node_group))
            
    def invoke(self, context, event):
        pass   

## END UI LIST
    
## PANEL

class SCENE_PT_scene_node_groups(bpy.types.Panel):
    '''Scene Node Groups Panel
    '''
    bl_label = bl_info["name"]
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Node Groups'


    def draw(self, context):
        '''Panel drawing
           :arg self:
           :tye self: SCENE_PT_node_groups_list
           :arg context: context
           :tye context: context
        '''
        scene = context.scene
        layout = self.layout
        col = layout.column()

        if scene.update_node_groups:
            col.prop(scene, "update_node_groups", toggle=True, icon='FILE_REFRESH')
        col.template_list("CUSTOM_UL_scene_node_groups",
                          "", 
                          scene,
                          "node_groups_list",
                          scene,
                          "node_groups_list_active_index")

        groups = list(bpy.data.node_groups)
        if groups:
            val='GRP_0'
        else:
            val='GRP'
        col.label(text='col.prop_enum')
        col.prop_enum(scene,'node_groups_enumprop',val)
        
        col.label(text='col.prop')
        col.prop(scene,'node_groups_enumprop')
        


## END PANEL

classes = [
    SceneNodeGroupsPropGroup,
    CUSTOM_UL_scene_node_groups,
    SCENE_PT_scene_node_groups
]

class_register, class_unregister = bpy.utils.register_classes_factory(classes)


def register():
    
    class_register()
    
    
    # Scene Node Groups Enumeration Property
    bpy.types.Scene.node_groups_enumprop = bpy.props.EnumProperty(items=get_all_node_groups,
                                                                  name='Node Groups',
                                                                  description='Scene Node Groups',
                                                                  default = 0,
                                                                  options={'SKIP_SAVE'}                                                                  
                                                                  )
#                                                                  update=update_node_group_enum,
#                                                                  get=get_node_group_enum,
#                                                                  set=set_node_group_enum,
#                                                                  )
    
    # Scene Node Groups Property
    bpy.types.Scene.node_groups = property(get_scene_node_groups)

    # Scene Update Node Groups Boolean Property for Panel
    bpy.types.Scene.update_node_groups = bpy.props.BoolProperty(
        get=get_all_data_node_groups,
        set=set_all_data_node_groups,
        name="Update Scene Node Groups")

    # Scene Node Groups Index Property
    bpy.types.Scene.node_groups_list_active_index = bpy.props.IntProperty()
    
    # Scene Collection Property for UI List 
    bpy.types.Scene.node_groups_list = bpy.props.CollectionProperty(type=SceneNodeGroupsPropGroup)

def unregister():

    class_unregister()
    
    del bpy.types.Scene.node_groups_enumprop
    del bpy.types.Scene.node_groups_list_active_index
    del bpy.types.Scene.node_groups_list
    del bpy.types.Scene.update_node_groups 
    del bpy.types.Scene.node_groups


if __name__ == "__main__":
    register()
